@@ -34,7 +34,7 @@ import math
 from urllib.request import urlretrieve
 from importlib import util
 
-program_version = "1.1.4"
+program_version = "1.1.6"
 eth_addr = "0x81F7B979fEa6134bA9FD5c701b3501A2e61E897a"
 btc_addr = "3CPmacGm34qYvR6XWLVEJmi2aNe3PZqUuq"
 
@@ -183,11 +183,9 @@ mcus = {
 models = {
     0xA4: [410000000, 525000000, 14, "410 - 525 MHz", "rnode_firmware_latest.hex"],
     0xA9: [820000000, 1020000000, 17, "820 - 1020 MHz", "rnode_firmware_latest.hex"],
-    0xF4: [410000000, 525000000, 14, "410 - 525 MHz", "rnode_firmware_latest_m2560.hex"],
-    0xF9: [820000000, 1020000000, 17, "820 - 1020 MHz", "rnode_firmware_latest_m2560.hex"],
     0xE4: [420000000, 520000000, 14, "420 - 520 MHz", "rnode_firmware_latest_tbeam.zip"],
     0xE9: [850000000, 950000000, 17, "850 - 950 MHz", "rnode_firmware_latest_tbeam.zip"],
-    0xFF: [100000000, 1100000000, 14, "(Band capabilities unknown)"],
+    0xFF: [100000000, 1100000000, 14, "(Band capabilities unknown)", None],
 }
 
 squashvw = False
@@ -829,38 +827,56 @@ def main():
             global squashvw
             squashvw = True
 
-            ports = list_ports.comports()
-            portlist = []
-            for port in ports:
-                portlist.insert(0, port) 
-            
-            pi = 1
-            print("Detected serial ports:")
-            for port in portlist:
-                print("  ["+str(pi)+"] "+str(port.device)+" ("+str(port.product)+", "+str(port.serial_number)+")")
-                pi += 1
-
-            print("\nWhat serial port is your device connected to? ", end="")
             selected_port = None
-            try:
-                c_port = int(input())
-                if c_port < 1 or c_port > len(ports):
-                    raise ValueError()
+            if not args.port:
+                ports = list_ports.comports()
+                portlist = []
+                for port in ports:
+                    portlist.insert(0, port) 
+                
+                pi = 1
+                print("Detected serial ports:")
+                for port in portlist:
+                    print("  ["+str(pi)+"] "+str(port.device)+" ("+str(port.product)+", "+str(port.serial_number)+")")
+                    pi += 1
 
-                selected_port = portlist[c_port-1]
-            except Exception as e:
-                print("That port does not exist, exiting now.")
-                exit()
+                print("\nWhat serial port is your device connected to? ", end="")
+                try:
+                    c_port = int(input())
+                    if c_port < 1 or c_port > len(ports):
+                        raise ValueError()
 
-            if selected_port == None:
-                print("Could not select port, exiting now.")
-                exit()
+                    selected_port = portlist[c_port-1]
+                except Exception as e:
+                    print("That port does not exist, exiting now.")
+                    exit()
 
-            port_path = selected_port.device
-            port_product = selected_port.product
-            port_serialno = selected_port.serial_number
+                if selected_port == None:
+                    print("Could not select port, exiting now.")
+                    exit()
 
-            print("\nOk, using device on "+str(port_path)+" ("+str(port_product)+", "+str(port_serialno)+")")
+                port_path = selected_port.device
+                port_product = selected_port.product
+                port_serialno = selected_port.serial_number
+
+                print("\nOk, using device on "+str(port_path)+" ("+str(port_product)+", "+str(port_serialno)+")")
+
+            else:
+                ports = list_ports.comports()
+
+                for port in ports:
+                    if port.device == args.port:
+                        selected_port = port
+
+                if selected_port == None:
+                    print("Could not find specified port "+str(args.port)+", exiting now")
+                    exit()
+
+                port_path = selected_port.device
+                port_product = selected_port.product
+                port_serialno = selected_port.serial_number
+
+                print("\nUsing device on "+str(port_path))
 
             print("\nProbing device...")
 
@@ -913,13 +929,25 @@ def main():
                     selected_product = ROM.PRODUCT_RNODE
                 elif c_dev == 2:
                     selected_product = ROM.PRODUCT_HMBRW
-                elif c_dev == 3:
-                    selected_product = ROM.PRODUCT_TBEAM
-                    print("\nImportant! Using RNode firmware on T-Beam devices should currently be")
+                    print("")
+                    print("---------------------------------------------------------------------------")
+                    print("Important! Using RNode firmware on homebrew devices should currently be")
                     print("considered experimental. It is not intended for production or critical use.")
                     print("The currently supplied firmware is provided AS-IS as a courtesey to those")
                     print("who would like to experiment with it. If you want any degree of reliability,")
                     print("please use an actual RNode from unsigned.io. Hit enter to continue.")
+                    print("---------------------------------------------------------------------------")
+                    input()
+                elif c_dev == 3:
+                    selected_product = ROM.PRODUCT_TBEAM
+                    print("")
+                    print("---------------------------------------------------------------------------")
+                    print("Important! Using RNode firmware on T-Beam devices should currently be")
+                    print("considered experimental. It is not intended for production or critical use.")
+                    print("The currently supplied firmware is provided AS-IS as a courtesey to those")
+                    print("who would like to experiment with it. If you want any degree of reliability,")
+                    print("please use an actual RNode from unsigned.io. Hit enter to continue.")
+                    print("---------------------------------------------------------------------------")
                     input()
             except Exception as e:
                 print("That device type does not exist, exiting now.")
@@ -1004,13 +1032,26 @@ def main():
                     if selected_mcu == ROM.MCU_1284P:
                         fw_filename = "rnode_firmware_latest.hex"
                     elif selected_mcu == ROM.MCU_2560:
-                        # This variant is not released yet
-                        #fw_filename = "rnode_firmware_latest_m2560.hex"
-                        fw_filename = None
+                        fw_filename = "rnode_firmware_latest_m2560.hex"
                 elif selected_platform == ROM.PLATFORM_ESP32:
                     # This variant is not released yet
-                    #fw_filename = "rnode_firmware_latest_esp32.zip"
+                    #fw_filename = "rnode_firmware_latest_esp32_generic.zip"
                     fw_filename = None
+                    print("\nWhat kind of ESP32 board is this?\n")
+                    print("[1] Adafruit Feather ESP32 (HUZZAH32)")
+                    print("[2] Generic ESP32 board")
+                    print("\n? ", end="")
+                    try:
+                        c_eboard = int(input())
+                        if c_eboard < 1 or c_eboard > 2:
+                            raise ValueError()
+                        elif c_eboard == 1:
+                            fw_filename = "rnode_firmware_latest_featheresp32.zip"
+                        elif c_eboard == 2:
+                            fw_filename = "rnode_firmware_latest_esp32_generic.zip"
+                    except Exception as e:
+                        print("That ESP32 board does not exist, exiting now.")
+                        exit()
             
             if fw_filename == None:
                 print("")
@@ -1148,7 +1189,12 @@ def main():
             elif platform == ROM.PLATFORM_AVR:
                 flasher = "avrdude"
                 if which(flasher) is not None:
-                    return [flasher, "-P", args.port, "-p", "m1284p", "-c", "arduino", "-b", "115200", "-U", "flash:w:update/"+fw_filename]
+                    # avrdude -C/home/markqvist/.arduino15/packages/arduino/tools/avrdude/6.3.0-arduino17/etc/avrdude.conf -q -q -V -patmega2560 -cwiring -P/dev/ttyACM0 -b115200 -D -Uflash:w:/tmp/arduino-sketch-0E260F46C421A84A7CBAD48E859C8E64/RNode_Firmware.ino.hex:i
+                    # avrdude -q -q -V -patmega2560 -cwiring -P/dev/ttyACM0 -b115200 -D -Uflash:w:/tmp/arduino-sketch-0E260F46C421A84A7CBAD48E859C8E64/RNode_Firmware.ino.hex:i
+                    if fw_filename == "rnode_firmware_latest.hex":
+                        return [flasher, "-P", args.port, "-p", "m1284p", "-c", "arduino", "-b", "115200", "-U", "flash:w:update/"+fw_filename+":i"]
+                    elif fw_filename == "rnode_firmware_latest_m2560.hex":
+                        return [flasher, "-P", args.port, "-p", "atmega2560", "-c", "wiring", "-D", "-b", "115200", "-U", "flash:w:update/"+fw_filename]
                 else:
                     RNS.log("")
                     RNS.log("You do not currently have the \""+flasher+"\" program installed on your system.")
@@ -1162,22 +1208,59 @@ def main():
             elif platform == ROM.PLATFORM_ESP32:
                 flasher = "./update/esptool.py" 
                 if which(flasher) is not None:
-                    return [
-                        flasher,
-                        "--chip", "esp32",
-                        "--port", args.port,
-                        "--baud", "921600",
-                        "--before", "default_reset",
-                        "--after", "hard_reset",
-                        "write_flash", "-z",
-                        "--flash_mode", "dio",
-                        "--flash_freq", "80m",
-                        "--flash_size", "4MB",
-                        "0xe000", "./update/rnode_firmware_latest_tbeam.boot_app0",
-                        "0x1000", "./update/rnode_firmware_latest_tbeam.bootloader",
-                        "0x10000", "./update/rnode_firmware_latest_tbeam.bin",
-                        "0x8000", "./update/rnode_firmware_latest_tbeam.partitions",
-                    ]
+                    if fw_filename == "rnode_firmware_latest_tbeam.zip":
+                        return [
+                            flasher,
+                            "--chip", "esp32",
+                            "--port", args.port,
+                            "--baud", "921600",
+                            "--before", "default_reset",
+                            "--after", "hard_reset",
+                            "write_flash", "-z",
+                            "--flash_mode", "dio",
+                            "--flash_freq", "80m",
+                            "--flash_size", "4MB",
+                            "0xe000", "./update/rnode_firmware_latest_tbeam.boot_app0",
+                            "0x1000", "./update/rnode_firmware_latest_tbeam.bootloader",
+                            "0x10000", "./update/rnode_firmware_latest_tbeam.bin",
+                            "0x8000", "./update/rnode_firmware_latest_tbeam.partitions",
+                        ]
+                    elif fw_filename == "rnode_firmware_latest_featheresp32.zip":
+                        return [
+                            flasher,
+                            "--chip", "esp32",
+                            "--port", args.port,
+                            "--baud", "921600",
+                            "--before", "default_reset",
+                            "--after", "hard_reset",
+                            "write_flash", "-z",
+                            "--flash_mode", "dio",
+                            "--flash_freq", "80m",
+                            "--flash_size", "4MB",
+                            "0xe000", "./update/rnode_firmware_latest_featheresp32.boot_app0",
+                            "0x1000", "./update/rnode_firmware_latest_featheresp32.bootloader",
+                            "0x10000", "./update/rnode_firmware_latest_featheresp32.bin",
+                            "0x8000", "./update/rnode_firmware_latest_featheresp32.partitions",
+                        ]
+                    elif fw_filename == "rnode_firmware_latest_esp32_generic.zip":
+                        return [
+                            flasher,
+                            "--chip", "esp32",
+                            "--port", args.port,
+                            "--baud", "921600",
+                            "--before", "default_reset",
+                            "--after", "hard_reset",
+                            "write_flash", "-z",
+                            "--flash_mode", "dio",
+                            "--flash_freq", "80m",
+                            "--flash_size", "4MB",
+                            "0xe000", "./update/rnode_firmware_latest_esp32_generic.boot_app0",
+                            "0x1000", "./update/rnode_firmware_latest_esp32_generic.bootloader",
+                            "0x10000", "./update/rnode_firmware_latest_esp32_generic.bin",
+                            "0x8000", "./update/rnode_firmware_latest_esp32_generic.partitions",
+                        ]
+                    else:
+                        RNS.log("No flasher available for this board, cannot install firmware.")
                 else:
                     RNS.log("")
                     RNS.log("You do not currently have the \""+flasher+"\" program installed on your system.")
@@ -1279,9 +1362,7 @@ def main():
                         if rnode.mcu == ROM.MCU_1284P:
                             fw_filename = "rnode_firmware_latest.hex"
                         elif rnode.mcu == ROM.MCU_2560:
-                            # This variant is not released yet
-                            #fw_filename = "rnode_firmware_latest_m2560.hex"
-                            fw_filename = None
+                            fw_filename = "rnode_firmware_latest_m2560.hex"
                     elif rnode.platform == ROM.PLATFORM_ESP32:
                         # This variant is not released yet
                         #fw_filename = "rnode_firmware_latest_esp32.zip"
@@ -1494,10 +1575,11 @@ def main():
                             model = ROM.MODEL_A4
                         elif args.model == "a9":
                             model = ROM.MODEL_A9
-                        elif args.model == "f4":
-                            model = ROM.MODEL_F4
-                        elif args.model == "f9":
-                            model = ROM.MODEL_F9
+                        # TODO: Remove, no more homebrew model differentiation
+                        # elif args.model == "f4":
+                        #     model = ROM.MODEL_F4
+                        # elif args.model == "f9":
+                        #     model = ROM.MODEL_F9
                         elif args.model == "e4":
                             model = ROM.MODEL_E4
                         elif args.model == "e9":
